import { Construct } from 'constructs';
import { Include } from './include';
/**
 * (experimental) Options for `Helm`.
 *
 * @experimental
 */
export interface HelmProps {
    /**
     * (experimental) The chart name to use. It can be a chart from a helm repository or a local directory.
     *
     * This name is passed to `helm template` and has all the relevant semantics.
     *
     * @experimental
     * @example
     *
     * "bitnami/redis"
     */
    readonly chart: string;
    /**
     * (experimental) The release name.
     *
     * @default - if unspecified, a name will be allocated based on the construct path
     * @see https://helm.sh/docs/intro/using_helm/#three-big-concepts
     * @experimental
     */
    readonly releaseName?: string;
    /**
     * (experimental) Values to pass to the chart.
     *
     * @default - If no values are specified, chart will use the defaults.
     * @experimental
     */
    readonly values?: {
        [key: string]: any;
    };
    /**
     * (experimental) The local helm executable to use in order to create the manifest the chart.
     *
     * @default "helm"
     * @experimental
     */
    readonly helmExecutable?: string;
    /**
     * (experimental) Additional flags to add to the `helm` execution.
     *
     * @default []
     * @experimental
     */
    readonly helmFlags?: string[];
}
/**
 * (experimental) Represents a Helm deployment.
 *
 * Use this construct to import an existing Helm chart and incorporate it into your constructs.
 *
 * @experimental
 */
export declare class Helm extends Include {
    /**
     * (experimental) The helm release name.
     *
     * @experimental
     */
    readonly releaseName: string;
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: HelmProps);
}
